@@ -10,11 +10,9 @@ import datetime
 import numpy as np
 import scipy as sp
 import scipy.stats
-import warnings
 import sys
 from copy import deepcopy
 from itertools import combinations
-from collections import deque
 from sklearn.metrics import get_scorer
 from sklearn.base import clone
 from sklearn.base import BaseEstimator
@@ -45,7 +43,7 @@ class SequentialFeatureSelector(BaseEstimator, MetaEstimatorMixin):
     Parameters
     ----------
     estimator : scikit-learn classifier or regressor
-    k_features : int or tuple (new in 0.4.2) (default: 1)
+    k_features : int or tuple or str (default: 1)
         Number of features to select,
         where k_features < the full feature set.
         New in 0.4.2: A tuple containing a min and max value can be provided,
@@ -53,6 +51,12 @@ class SequentialFeatureSelector(BaseEstimator, MetaEstimatorMixin):
             min and max that scored highest in cross-validtion. For example,
             the tuple (1, 4) will return any combination from
             1 up to 4 features instead of a fixed number of features k.
+        New in 0.8.0: A string argument "best" or "parsimonious".
+            If "best" is provided, the feature selector will return the
+            feature subset with the best cross-validation performance.
+            If "parsimonious" is provided as an argument, the smallest
+            feature subset that is within one standard error of the
+            cross-validation performance will be selected.
     forward : bool (default: True)
         Forward selection if True,
         backward selection otherwise
@@ -79,10 +83,6 @@ class SequentialFeatureSelector(BaseEstimator, MetaEstimatorMixin):
         stratified k-fold is performed, and regular k-fold cross-validation
         otherwise.
         No cross-validation if cv is None, False, or 0.
-    skip_if_stuck: bool (default: True)
-        Set to True to skip conditional
-        exclusion/inclusion if floating=True and
-        algorithm gets stuck in cycles.
     n_jobs : int (default: 1)
         The number of CPUs to use for evaluating different feature subsets
         in parallel. -1 means 'all CPUs'.
@@ -123,7 +123,7 @@ class SequentialFeatureSelector(BaseEstimator, MetaEstimatorMixin):
     def __init__(self, estimator, k_features=1,
                  forward=True, floating=False,
                  verbose=0, scoring=None,
-                 cv=5, skip_if_stuck=True, n_jobs=1,
+                 cv=5, n_jobs=1,
                  pre_dispatch='2*n_jobs',
                  clone_estimator=True):
 
@@ -136,7 +136,6 @@ class SequentialFeatureSelector(BaseEstimator, MetaEstimatorMixin):
         self.n_jobs = n_jobs
         self.named_est = {key: value for key, value in
                           _name_estimators([self.estimator])}
-        self.skip_if_stuck = skip_if_stuck
         self.cv = cv
         self.n_jobs = n_jobs
         self.verbose = verbose
@@ -186,11 +185,11 @@ class SequentialFeatureSelector(BaseEstimator, MetaEstimatorMixin):
         self : object
 
         """
-
         if not isinstance(self.k_features, int) and\
-                not isinstance(self.k_features, tuple):
-            raise AttributeError('k_features must be a positive integer'
-                                 ' or tuple')
+                not isinstance(self.k_features, tuple)\
+                and not isinstance(self.k_features, str):
+                raise AttributeError('k_features must be a positive integer'
+                                     ', tuple, or string')
 
         if isinstance(self.k_features, int) and (self.k_features < 1 or
                                                  self.k_features > X.shape[1]):
@@ -212,16 +211,25 @@ class SequentialFeatureSelector(BaseEstimator, MetaEstimatorMixin):
                                      ' range(1, X.shape[1]+1).')
 
             if self.k_features[0] > self.k_features[1]:
-                raise AttributeError('The min k_features value must be larger'
+                raise AttributeError('The min k_features value must be smaller'
                                      ' than the max k_features value.')
 
-        if self.skip_if_stuck:
-            sdq = deque(maxlen=4)
-        else:
-            sdq = deque(maxlen=0)
+        if isinstance(self.k_features, tuple) or\
+                isinstance(self.k_features, str):
 
-        if isinstance(self.k_features, tuple):
             select_in_range = True
+
+            if isinstance(self.k_features, str):
+                if self.k_features not in {'best', 'parsimonious'}:
+                    raise AttributeError('If a string argument is provided, '
+                                         'it must be "best" or "parsimonious"')
+                else:
+                    min_k = 1
+                    max_k = X.shape[1]
+            else:
+                min_k = self.k_features[0]
+                max_k = self.k_features[1]
+
         else:
             select_in_range = False
             k_to_select = self.k_features
@@ -230,26 +238,27 @@ class SequentialFeatureSelector(BaseEstimator, MetaEstimatorMixin):
         orig_set = set(range(X.shape[1]))
         if self.forward:
             if select_in_range:
-                k_to_select = self.k_features[1]
+                k_to_select = max_k
             k_idx = ()
             k = 0
         else:
             if select_in_range:
-                k_to_select = self.k_features[0]
+                k_to_select = min_k
             k_idx = tuple(range(X.shape[1]))
             k = len(k_idx)
             k_idx, k_score = _calc_score(self, X, y, k_idx)
             self.subsets_[k] = {
                 'feature_idx': k_idx,
                 'cv_scores': k_score,
-                'avg_score': k_score.mean()
+                'avg_score': np.nanmean(k_score)
             }
-
         best_subset = None
         k_score = 0
+
         try:
             while k != k_to_select:
                 prev_subset = set(k_idx)
+
                 if self.forward:
                     k_idx, k_score, cv_scores = self._inclusion(
                         orig_set=orig_set,
@@ -258,14 +267,17 @@ class SequentialFeatureSelector(BaseEstimator, MetaEstimatorMixin):
                         y=y
                     )
                 else:
+
                     k_idx, k_score, cv_scores = self._exclusion(
                         feature_set=prev_subset,
                         X=X,
                         y=y
                     )
 
-                if self.floating and not self._is_stuck(sdq):
+                if self.floating:
+                    k_score_c = None
                     (new_feature,) = set(k_idx) ^ prev_subset
+
                     if self.forward:
                         k_idx_c, k_score_c, cv_scores_c = self._exclusion(
                             feature_set=k_idx,
@@ -273,6 +285,7 @@ class SequentialFeatureSelector(BaseEstimator, MetaEstimatorMixin):
                             X=X,
                             y=y
                         )
+
                     else:
                         k_idx_c, k_score_c, cv_scores_c = self._inclusion(
                             orig_set=orig_set - {new_feature},
@@ -281,24 +294,28 @@ class SequentialFeatureSelector(BaseEstimator, MetaEstimatorMixin):
                             y=y
                         )
 
-                    if k_score_c and k_score_c > k_score:
-                        k_idx, k_score, cv_scores = \
-                            k_idx_c, k_score_c, cv_scores_c
+                    if k_score_c is not None and k_score_c > k_score:
+
+                        if len(k_idx_c) in self.subsets_:
+                            cached_score = self.subsets_[len(
+                                k_idx_c)]['avg_score']
+                        else:
+                            cached_score = None
+
+                        if cached_score is None or k_score_c > cached_score:
+                            k_idx, k_score, cv_scores = \
+                                k_idx_c, k_score_c, cv_scores_c
 
                 k = len(k_idx)
                 # floating can lead to multiple same-sized subsets
-                if k not in self.subsets_ or (self.subsets_[k]['avg_score'] <
-                                              k_score):
+                if k not in self.subsets_ or (k_score >
+                                              self.subsets_[k]['avg_score']):
                     self.subsets_[k] = {
                         'feature_idx': k_idx,
                         'cv_scores': cv_scores,
                         'avg_score': k_score
                     }
 
-                # k_idx must be a set otherwise different permutations
-                # would not be found as equal in _is_stuck
-                sdq.append(set(k_idx))
-
                 if self.verbose == 1:
                     sys.stderr.write('\rFeatures: %d/%s' % (
                         len(k_idx),
@@ -321,27 +338,37 @@ class SequentialFeatureSelector(BaseEstimator, MetaEstimatorMixin):
             sys.stderr.write('\nSTOPPING EARLY DUE TO KEYBOARD INTERRUPT...')
 
         if select_in_range:
+            max_score = float('-inf')
+
             max_score = float('-inf')
             for k in self.subsets_:
+                if k < min_k or k > max_k:
+                    continue
                 if self.subsets_[k]['avg_score'] > max_score:
                     max_score = self.subsets_[k]['avg_score']
                     best_subset = k
             k_score = max_score
             k_idx = self.subsets_[best_subset]['feature_idx']
 
+            if self.k_features == 'parsimonious':
+                for k in self.subsets_:
+                    if k >= best_subset:
+                        continue
+                    if self.subsets_[k]['avg_score'] >= (
+                            max_score - np.std(self.subsets_[k]['cv_scores']) /
+                            self.subsets_[k]['cv_scores'].shape[0]):
+                        max_score = self.subsets_[k]['avg_score']
+                        best_subset = k
+                k_score = max_score
+                k_idx = self.subsets_[best_subset]['feature_idx']
+
         self.k_feature_idx_ = k_idx
         self.k_score_ = k_score
         self.subsets_plus_ = dict()
         self.fitted = True
         return self
 
-    def _is_stuck(self, sdq):
-        stuck = False
-        if len(sdq) == 4 and (sdq[0] == sdq[2] or sdq[1] == sdq[3]):
-            stuck = True
-        return stuck
-
-    def _inclusion(self, orig_set, subset, X, y):
+    def _inclusion(self, orig_set, subset, X, y, ignore_feature=None):
         all_avg_scores = []
         all_cv_scores = []
         all_subsets = []
@@ -354,10 +381,11 @@ class SequentialFeatureSelector(BaseEstimator, MetaEstimatorMixin):
                                 pre_dispatch=self.pre_dispatch)
             work = parallel(delayed(_calc_score)
                             (self, X, y, tuple(subset | {feature}))
-                            for feature in remaining)
+                            for feature in remaining
+                            if feature != ignore_feature)
 
             for new_subset, cv_scores in work:
-                all_avg_scores.append(cv_scores.mean())
+                all_avg_scores.append(np.nanmean(cv_scores))
                 all_cv_scores.append(cv_scores)
                 all_subsets.append(new_subset)
 
@@ -383,7 +411,7 @@ class SequentialFeatureSelector(BaseEstimator, MetaEstimatorMixin):
                             if not fixed_feature or fixed_feature in set(p))
 
             for p, cv_scores in work:
-                all_avg_scores.append(cv_scores.mean())
+                all_avg_scores.append(np.nanmean(cv_scores))
                 all_cv_scores.append(cv_scores)
                 all_subsets.append(p)
 
